

set -ex



b2 --help
bjam --help
pushd tests
pushd hello_world_cxx
./test.sh
popd
popd
! test -f $PREFIX/include/boost/python.hpp
! test -d $PREFIX/include/boost/python
test -f $PREFIX/include/boost/assert.hpp
test -f $PREFIX/lib/libboost_exception.a
test -f $PREFIX/lib/libboost_test_exec_monitor.a
test -f $PREFIX/lib/libboost_atomic.a
test -f $PREFIX/lib/libboost_atomic.dylib
test -f $PREFIX/lib/libboost_chrono.a
test -f $PREFIX/lib/libboost_chrono.dylib
test -f $PREFIX/lib/libboost_container.a
test -f $PREFIX/lib/libboost_container.dylib
test -f $PREFIX/lib/libboost_context.a
test -f $PREFIX/lib/libboost_context.dylib
test -f $PREFIX/lib/libboost_coroutine.a
test -f $PREFIX/lib/libboost_coroutine.dylib
test -f $PREFIX/lib/libboost_date_time.a
test -f $PREFIX/lib/libboost_date_time.dylib
test -f $PREFIX/lib/libboost_filesystem.a
test -f $PREFIX/lib/libboost_filesystem.dylib
test -f $PREFIX/lib/libboost_graph.a
test -f $PREFIX/lib/libboost_graph.dylib
test -f $PREFIX/lib/libboost_iostreams.a
test -f $PREFIX/lib/libboost_iostreams.dylib
test -f $PREFIX/lib/libboost_locale.a
test -f $PREFIX/lib/libboost_locale.dylib
test -f $PREFIX/lib/libboost_log.a
test -f $PREFIX/lib/libboost_log.dylib
test -f $PREFIX/lib/libboost_log_setup.a
test -f $PREFIX/lib/libboost_log_setup.dylib
test -f $PREFIX/lib/libboost_math_c99.a
test -f $PREFIX/lib/libboost_math_c99.dylib
test -f $PREFIX/lib/libboost_math_c99f.a
test -f $PREFIX/lib/libboost_math_c99f.dylib
test -f $PREFIX/lib/libboost_math_c99l.a
test -f $PREFIX/lib/libboost_math_c99l.dylib
test -f $PREFIX/lib/libboost_math_tr1.a
test -f $PREFIX/lib/libboost_math_tr1.dylib
test -f $PREFIX/lib/libboost_math_tr1f.a
test -f $PREFIX/lib/libboost_math_tr1f.dylib
test -f $PREFIX/lib/libboost_math_tr1l.a
test -f $PREFIX/lib/libboost_math_tr1l.dylib
test -f $PREFIX/lib/libboost_prg_exec_monitor.a
test -f $PREFIX/lib/libboost_prg_exec_monitor.dylib
test -f $PREFIX/lib/libboost_program_options.a
test -f $PREFIX/lib/libboost_program_options.dylib
test -f $PREFIX/lib/libboost_random.a
test -f $PREFIX/lib/libboost_random.dylib
test -f $PREFIX/lib/libboost_regex.a
test -f $PREFIX/lib/libboost_regex.dylib
test -f $PREFIX/lib/libboost_serialization.a
test -f $PREFIX/lib/libboost_serialization.dylib
test -f $PREFIX/lib/libboost_system.a
test -f $PREFIX/lib/libboost_system.dylib
test -f $PREFIX/lib/libboost_thread.a
test -f $PREFIX/lib/libboost_thread.dylib
test -f $PREFIX/lib/libboost_timer.a
test -f $PREFIX/lib/libboost_timer.dylib
test -f $PREFIX/lib/libboost_type_erasure.a
test -f $PREFIX/lib/libboost_type_erasure.dylib
test -f $PREFIX/lib/libboost_unit_test_framework.a
test -f $PREFIX/lib/libboost_unit_test_framework.dylib
test -f $PREFIX/lib/libboost_wave.a
test -f $PREFIX/lib/libboost_wave.dylib
test -f $PREFIX/lib/libboost_wserialization.a
test -f $PREFIX/lib/libboost_wserialization.dylib
exit 0
