/*
Copyright 2010 Beman Dawes

Copyright 2019 Glen Joseph Fernandes
(glenjofe@gmail.com)

Distributed under the Boost Software License, Version 1.0.
(http://www.boost.org/LICENSE_1_0.txt)
*/
#ifndef BOOST_IO_DETAIL_QUOTED_MANIP_HPP
#define BOOST_IO_DETAIL_QUOTED_MANIP_HPP

#include <boost/config/header_deprecated.hpp>

BOOST_HEADER_DEPRECATED("<boost/io/quoted.hpp>")

#include <boost/io/quoted.hpp>

#endif
